@@ -81,6 +81,7 @@ namespace Lucene.Net.Store
 
 				bool obtainedLock = false;
 
+#if false
 				int fd = Mono.Posix.Syscall.open (lockFile.FullName,
 								  Mono.Posix.OpenFlags.O_CREAT
 								  | Mono.Posix.OpenFlags.O_EXCL,
@@ -92,12 +93,14 @@ namespace Lucene.Net.Store
 					lockFile.Refresh ();
 					obtainedLock = true;
 				}
+#endif
 			
-				
-#if false
 				try
 				{
 					System.IO.FileStream createdFile = lockFile.Create();
+					System.IO.StreamWriter writer = new System.IO.StreamWriter (createdFile);
+					writer.WriteLine (Process.GetCurrentProcess().Id);
+					writer.Close ();
 					createdFile.Close();
 					obtainedLock = true;
 				}
@@ -105,7 +108,6 @@ namespace Lucene.Net.Store
 				{
 					// Just fall through
 				}
-#endif
 
 				Log ("{0} lock {1}", obtainedLock ? "Obtained" : "Could not obtain", lockFile.FullName);
 				return obtainedLock;
@@ -502,7 +504,9 @@ namespace Lucene.Net.Store
                     try
                     {
                         in_Renamed = new System.IO.BinaryReader(System.IO.File.Open(old.FullName, System.IO.FileMode.Open, System.IO.FileAccess.Read));
-                        out_Renamed = new System.IO.FileStream(nu.FullName, System.IO.FileMode.Create);
+			// FIXED trow@novell.com 18 Jul 2005
+			// Open the FileStream w/ FileShare.ReadWrite.
+                        out_Renamed = new System.IO.FileStream(nu.FullName, System.IO.FileMode.Create, System.IO.FileAccess.Write, System.IO.FileShare.ReadWrite);
                         // see if the buffer needs to be initialized. Initialization is
                         // only done on-demand since many VM's will never run into the renameTo
                         // bug and hence shouldn't waste 1K of mem for no reason.
