@@ -31,6 +31,7 @@ using System.IO;
 using System.Net;
 using System.Collections;
 using System.Threading;
+using System.Diagnostics;
 
 using System.Runtime.Remoting;
 using System.Runtime.Remoting.Channels;
@@ -51,7 +52,7 @@ namespace Beagle.WebService {
 		public static string web_rootDir = DEFAULT_XSP_ROOT;
 		public static string web_port = DEFAULT_XSP_PORT;		
 		public static bool web_start = true;
-		public static bool web_global;
+		public static bool web_global = false;
 
 		static Mono.ASPNET.ApplicationServer appServer = null;
 		//Both "/" and "/beagle" aliased to DEFAULT_XSP_ROOT only for BeagleXSP server
@@ -69,8 +70,15 @@ namespace Beagle.WebService {
 		
 		public static void Start()
 		{			
+			if (! web_start) {
+				Logger.Log.Warn("Beagle running with WebServices DISABLED\n");
+				return;
+			}
+				
 			try {
-				hostname = Dns.GetHostName();
+				IPHostEntry hostInfo = Dns.GetHostByName(Dns.GetHostName());
+				//Fully qualified DNS name of host:
+				hostname = hostInfo.HostName;
 				Logger.Log.Info("This Computer Hostname: " + hostname);
 			}
 			catch (Exception ex) 
@@ -80,7 +88,8 @@ namespace Beagle.WebService {
 				hostname = "localhost";
 			}
 			
-			web_global = Conf.WebServices.AllowGlobalAccess;
+			if (! web_global)
+				web_global = Conf.WebServices.AllowGlobalAccess;
 			
 			//start web-access server first
 			Logger.Log.Debug ("Starting WebBackEnd");
@@ -112,6 +121,26 @@ namespace Beagle.WebService {
 					//Mapping /beagle/kde3 to ExternalStringsHack.KdePrefix
 				if (Directory.Exists(ExternalStringsHack.KdePrefix))
 				xsp_param[5] += ",/beagle/kde3:" + ExternalStringsHack.KdePrefix;
+
+				string imgDir = PathFinder.StorageDir + "/img";
+				if (!Directory.Exists(imgDir))
+				{
+					Process pr = new Process ();
+					pr.StartInfo.UseShellExecute = true; 
+					pr.StartInfo.FileName = "mkdir"; 
+					pr.StartInfo.Arguments = imgDir;
+
+					try {		
+						pr.Start ();
+						pr.WaitForExit(); 	  				
+	  					pr.Close();
+	  					pr.Dispose();				
+					} 
+					catch (Exception e) { 
+						Logger.Log.Warn("Error creating ~/.beagle/img folder"); 
+					} 				
+				}
+				xsp_param[5] += ",/beagle/img:" + imgDir;
 				
 				//if (!hostname.Equals("localhost")) {
 
@@ -131,29 +160,26 @@ namespace Beagle.WebService {
 			{
 				xsp_param[5] = DEFAULT_APP_MAPPINGS;
 			}					
-					     						
-			if (web_start) {
 				
-				Logger.Log.Debug ("Starting Internal Web Server");
+			Logger.Log.Debug ("Starting Internal Web Server");
 
-				int retVal = 0;
-				try {
-					//Start beagled internal web server (BeagleXsp)
-					retVal = Mono.ASPNET.Server.initXSP(xsp_param, out appServer);
-				}
-				catch (ArgumentException e) {
-					//Retry with default application mappings:
-					xsp_param[5] = DEFAULT_APP_MAPPINGS;
-					retVal = Mono.ASPNET.Server.initXSP(xsp_param, out appServer);		
-				}
-
-				if (retVal != 0) {
-					Logger.Log.Warn ("Error starting Internal Web Server (retVal={0})", retVal);
-					Logger.Log.Warn ("Check if there is another instance of Beagle running");
-				}
-				else
-					Logger.Log.Debug("BeagleXSP Applications list: " + xsp_param[5]);
+			int retVal = 0;
+			try {
+				//Start beagled internal web server (BeagleXsp)
+				retVal = Mono.ASPNET.Server.initXSP(xsp_param, out appServer);
 			}
+			catch (ArgumentException e) {
+				//Retry with default application mappings:
+				xsp_param[5] = DEFAULT_APP_MAPPINGS;
+				retVal = Mono.ASPNET.Server.initXSP(xsp_param, out appServer);		
+			}
+
+			if (retVal != 0) {
+				Logger.Log.Warn ("Error starting Internal Web Server (retVal={0})", retVal);
+				Logger.Log.Warn ("Check if there is another instance of Beagle running");
+			}
+			else
+				Logger.Log.Debug("BeagleXSP Applications list: " + xsp_param[5]);
 		}
 		
 		public static void Stop() 
@@ -163,6 +189,19 @@ namespace Beagle.WebService {
 			    appServer.Stop(); 
 				appServer = null;
 			}
+			
+			Process pr = new Process ();
+			pr.StartInfo.UseShellExecute = true; 
+			pr.StartInfo.FileName = "rm"; 
+			pr.StartInfo.Arguments = " -rf  " + PathFinder.StorageDir + "/img/*";
+
+			try {		
+				pr.Start ();
+				pr.WaitForExit(); 	  				
+	  			pr.Close();
+	  			pr.Dispose();					
+			} 
+			catch (Exception e) { } 			
 		}
 		
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -209,7 +248,7 @@ namespace Beagle.WebService {
 		}
 		
 /////////////////////////////////////////////////////////////////////////////////////////		
-				
+			
 		//KNV: If needed, we can convert this to a Singleton, adding a 
 		//	   static Factory method to get the singleton instance reference,
 		//	   so that front-end code always gets hold of same instance.
@@ -268,7 +307,7 @@ namespace Beagle.WebService {
 						//Query query = sdata.query;					
 					lock (results.SyncRoot) {
 						foreach (Hit h in hits) 
-							if (h.Uri.ToString().StartsWith(NetworkedBeagle.BeagleNetPrefix) ||
+							if (h.UriAsString.StartsWith(NetworkedBeagle.BeagleNetPrefix) ||
 											AccessFilter.FilterHit(h))
 								results.Add(h);
 					}
@@ -400,11 +439,14 @@ namespace Beagle.WebService {
 			}
 
 			Query query = new Query();
-						
-			foreach (string text in sreq.text) 
-				query.AddText(text);				
+
+			string searchString = "";						
+			foreach (string text in sreq.text)  {
+				query.AddText(text);
+				searchString += (searchString.Length == 0) ? text:" " + text;
+			}			
 			
-			Logger.Log.Info("WebServiceBackEnd: Received {0} WebService Query with search term: {1}", isLocalReq ? "Local":"External", query.QuotedText);
+			Logger.Log.Info("WebServiceBackEnd: Received {0} WebService Query with search term: {1}", isLocalReq ? "Local":"External", searchString.Trim());
 
 			if (sreq.mimeType != null && sreq.mimeType[0] != null)
 				foreach (string mtype in sreq.mimeType)
@@ -438,7 +480,7 @@ namespace Beagle.WebService {
 						//If request has traversed 5 nodes in reaching here, stop cascading. 
 						//Make it a Local Query.
 						query.RemoveDomain(sreq.qdomain);
-						query.AddDomain(QueryDomain.Local);
+						query.AddDomain(QueryDomain.System);
 				 	}
 				 					 	
 					if ((sr == null) && (sreq.searchId != 0) )
@@ -468,8 +510,8 @@ namespace Beagle.WebService {
 */
 			sessionTable.Add(searchToken, sdata);
 			
-			QueryDriver.DoQuery (query, qres);
-
+			QueryDriver.DoQueryLocal (query, qres);
+					
 			while (resultTable.Contains(qres) && (results.Count < MAX_RESULTS_PER_CALL) )
 				Thread.Sleep(100);
 
@@ -490,16 +532,17 @@ namespace Beagle.WebService {
 
 					string snippet; 
 						
-					Queryable queryable = h.SourceObject as Queryable;
+					//Queryable queryable = h.SourceObject as Queryable;
+					Queryable queryable = QueryDriver.GetQueryable (h.SourceObjectName);
+					
 					if (queryable == null)
 						snippet = "ERROR: hit.SourceObject is null, uri=" + h.Uri;
 					else
 						snippet = queryable.GetSnippet (ICollection2StringList(query.Text), h);											
 								
 					sr.hitResults[i] = new HitResult();
-					sr.hitResults[i].id = h.Id;
 					
-					hitUri = h.Uri.ToString();
+					hitUri = h.UriAsString;
 					if (isLocalReq || hitUri.StartsWith(NetworkedBeagle.BeagleNetPrefix))
 							sr.hitResults[i].uri = hitUri;
 					else
@@ -508,8 +551,7 @@ namespace Beagle.WebService {
 	        	    sr.hitResults[i].resourceType = h.Type;
 					sr.hitResults[i].mimeType = h.MimeType;
 					sr.hitResults[i].source = h.Source;
-					sr.hitResults[i].scoreRaw = h.ScoreRaw;
-					sr.hitResults[i].scoreMultiplier = h.ScoreMultiplier;
+					sr.hitResults[i].score = h.Score;				
 				
 					int plen = h.Properties.Count;
 					sr.hitResults[i].properties = new HitProperty[plen];
@@ -518,10 +560,12 @@ namespace Beagle.WebService {
 						sr.hitResults[i].properties[j] = new HitProperty();
 						sr.hitResults[i].properties[j].PKey = p.Key;
 						sr.hitResults[i].properties[j].PVal = p.Value;				
-						sr.hitResults[i].properties[j].IsKeyword = p.IsKeyword;				
+						sr.hitResults[i].properties[j].IsMutable = p.IsMutable;				
 						sr.hitResults[i].properties[j].IsSearched = p.IsSearched;							
 					}
 
+					sr.hitResults[i].hashCode = h.GetHashCode ();
+					
 					if (snippet != null)
 						sr.hitResults[i].snippet = snippet.Trim();
 				}					
@@ -547,16 +591,18 @@ namespace Beagle.WebService {
 			 return sr;
 		}
 
-		public SearchResult getMoreResults(string searchToken, int startIndex, bool isLocalReq)
-		{							
-
+		public SearchResult getResults(GetResultsRequest req, bool isLocalReq)
+		{										
+			int startIndex = req.startIndex;
+			string searchToken = req.searchToken;
+						
 			SearchResult sr = new SearchResult();
 			sr.numResults = 0;
 			
 			if (!sessionTable.ContainsKey(searchToken)) {
 				sr.statusCode = SC_INVALID_SEARCH_TOKEN;
 				sr.statusMsg = "Error: Invalid Search Token";
-				Logger.Log.Warn("GetMoreResults: Invalid Search Token received ");
+				Logger.Log.Warn("GetResults: Invalid Search Token received ");
 				return sr;
 			}
 									
@@ -564,7 +610,7 @@ namespace Beagle.WebService {
 			if (results == null) {
 				sr.statusCode = SC_INVALID_SEARCH_TOKEN;
 				sr.statusMsg = "Error: Invalid Search Token";
-				Logger.Log.Warn("GetMoreResults: Invalid Search Token received ");
+				Logger.Log.Warn("GetResults: Invalid Search Token received ");
 				return sr;
 			}
 
@@ -584,12 +630,10 @@ namespace Beagle.WebService {
 							
 					sr.hitResults[i] = new HitResult();
 					
-// GetMoreResults will NOT return Snippets by default. Client must make explicit GetSnippets request to get snippets for these hits.
+// GetResults will NOT return Snippets by default. Client must make explicit GetSnippets request to get snippets for these hits.
 // Not initializing sr.hitResults[i].snippet implies there is no <snippets> element in HitResult XML response.
-								
-					sr.hitResults[i].id = h.Id;
 							
-					hitUri = h.Uri.ToString();
+					hitUri = h.UriAsString;
 					if (isLocalReq || hitUri.StartsWith(NetworkedBeagle.BeagleNetPrefix))
 							sr.hitResults[i].uri = hitUri;
 					else
@@ -598,9 +642,8 @@ namespace Beagle.WebService {
 	        	    sr.hitResults[i].resourceType = h.Type;
 					sr.hitResults[i].mimeType = h.MimeType;
 					sr.hitResults[i].source = h.Source;
-					sr.hitResults[i].scoreRaw = h.ScoreRaw;
-					sr.hitResults[i].scoreMultiplier = h.ScoreMultiplier;
-				
+					sr.hitResults[i].score = h.Score;
+									
 					int plen = h.Properties.Count;
 					sr.hitResults[i].properties = new HitProperty[plen];
 					for (int j = 0; j < plen; j++) {
@@ -608,10 +651,12 @@ namespace Beagle.WebService {
 						sr.hitResults[i].properties[j] = new HitProperty();
 						sr.hitResults[i].properties[j].PKey = p.Key;
 						sr.hitResults[i].properties[j].PVal = p.Value;				
-						sr.hitResults[i].properties[j].IsKeyword = p.IsKeyword;				
+						sr.hitResults[i].properties[j].IsMutable = p.IsMutable;				
 						sr.hitResults[i].properties[j].IsSearched = p.IsSearched;							
 					}												
 					
+					sr.hitResults[i].hashCode = h.GetHashCode ();
+					
 					i++;
 				}												
 			} //end lock
@@ -631,9 +676,11 @@ namespace Beagle.WebService {
 		}
 		
 		public static string InvalidHitSnippetError = "ERROR: Invalid or Duplicate Hit Id";
-		public HitSnippet[] getSnippets(string searchToken, int[] hitIds)
+		public HitSnippet[] getSnippets(GetSnippetsRequest req)
 		{	
 			HitSnippet[] response;
+			string searchToken = req.searchToken;
+			int[] hitHashCodes = req.hitHashCodes;
 			
 			if (!sessionTable.ContainsKey(searchToken)) {
 			
@@ -651,10 +698,11 @@ namespace Beagle.WebService {
 			}
 			
 			int i = 0; 		
-			ArrayList IdList = new ArrayList();
-			IdList.AddRange(hitIds);	
-			response = new HitSnippet[hitIds.Length];
-			Logger.Log.Debug("GetSnippets invoked with {0} hitIds", hitIds.Length);
+			ArrayList hashCodeList = new ArrayList();
+			hashCodeList.AddRange(hitHashCodes);
+				
+			response = new HitSnippet[hitHashCodes.Length];
+			Logger.Log.Debug("GetSnippets invoked with {0} hitHashCodes", hitHashCodes.Length);
 
 			Query query = ((SessionData)sessionTable[searchToken]).query;
 			
@@ -663,11 +711,14 @@ namespace Beagle.WebService {
 				string snippet = null; 
 				foreach (Hit h in results)  {
 				
-					if (IdList.Contains(h.Id)) {
+					int hashCode = h.GetHashCode();
+					if (hashCodeList.Contains(hashCode)) {
 					
-							IdList.Remove(h.Id);	
-													
-							Queryable queryable = h.SourceObject as Queryable;
+							hashCodeList.Remove(hashCode);	
+
+							//Queryable queryable = h.SourceObject as Queryable;
+							Queryable queryable = QueryDriver.GetQueryable (h.SourceObjectName);													
+
 							if (queryable == null)
 								snippet = "ERROR: hit.SourceObject is null, uri=" + h.Uri;
 							else
@@ -678,23 +729,23 @@ namespace Beagle.WebService {
 								snippet = "";
 								
 							HitSnippet hs = new HitSnippet();
-							hs.hitId = h.Id; 
+							hs.hashCode = hashCode; 
 							hs.snippet = snippet.Trim();
 							response[i++] = hs;		
 					
-							if (i == hitIds.Length)
+							if ((hashCodeList.Count == 0) || (i == hitHashCodes.Length))
 								return response;
 					}
 				} //end foreach
 			} //end lock
 			
-			foreach (int hitId in IdList) {
+			foreach (int hashCode in hashCodeList) {
 					HitSnippet hs = new HitSnippet();
-					hs.hitId = hitId; 
+					hs.hashCode = hashCode; 
 					hs.snippet = InvalidHitSnippetError;
 					response[i++] = hs;	
 
-					if (i == hitIds.Length)
+					if (i == hitHashCodes.Length)
 							break;
 			}		
 			Logger.Log.Warn("GetSnippets invoked some invalid hitIds");
@@ -715,11 +766,15 @@ namespace Beagle.WebService {
 			return (token.Replace('-', alpha));
 		}
 	}	
-	
+
+
 ////////////////////////////////////////////////////////////////////////////////////////////
 /////////////   WebService Request-Response Data Structures   	
 ////////////////////////////////////////////////////////////////////////////////////////////
 
+/* These are duplicate definitions to the ones in WebServiceProxy.cs. 
+    So, we will define and use these from one central place: WebServiceProxy.cs
+
 	[Serializable()]
 	public class SearchRequest  {
 
@@ -747,10 +802,10 @@ namespace Beagle.WebService {
 			set {pVal = value;}
 		}
 		
-		private bool  isKeyword;
-		public bool IsKeyword {
-			get { return isKeyword; }
-			set { isKeyword = value; }
+		private bool  isMutable;
+		public bool IsMutable {
+			get { return isMutable; }
+			set { isMutable = value; }
 		}
 		
 		private bool   isSearched;
@@ -763,15 +818,15 @@ namespace Beagle.WebService {
 	[Serializable()]
 	public class HitResult {
 
-		public int 		id;
 		public string 	uri;
+		//public string 	parentUri;		
 		public string 	resourceType;
 		public string 	mimeType;
 		public string 	source;
-		public double 	scoreRaw;
-		public double 	scoreMultiplier;
+		public double 	score;
 		public HitProperty[] properties;
 		//FIXME: public xxx[] data;
+		public int 	hashCode;
 		public string 	snippet;
 	}
 
@@ -791,8 +846,9 @@ namespace Beagle.WebService {
 
 	[Serializable()]
 	public class HitSnippet {
-		public int hitId;
+		public int hashCode;
 		public string snippet;
 	}
+*/	
 		
 }
