@@ -39,7 +39,7 @@ class InfoTool {
 		usage +=
 			"Usage: beagle-info <OPTIONS>\n\n" +
 			"Options:\n" +
-			"  --daemon-version\t\t\tPrint the version of the running daemon.\n" +
+			"  --daemon-version\t\tPrint the version of the running daemon.\n" +
 			"  --status\t\t\tDisplay status of the running daemon.\n" +
 			"  --index-info\t\t\tDisplay statistics of the Beagle indexes.\n" +
 			"  --help\t\t\tPrint this usage message.\n";
@@ -59,7 +59,7 @@ class InfoTool {
 
 		try {
 			response = (DaemonInformationResponse) request.Send ();
-		} catch (System.Net.Sockets.SocketException) {
+		} catch (Beagle.ResponseMessageException) {
 			Console.WriteLine ("Could not connect to the daemon.");
 			return 1;
 		}
@@ -79,4 +79,4 @@ class InfoTool {
 	}
 }
 		
-		
\ No newline at end of file
+		
